#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                   $Date$
#                   $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
from __future__ import print_function
import xml.etree.cElementTree as ET
import datetime
import sys
from os import listdir
from os.path import isfile, join
import os.path
from tempfile import NamedTemporaryFile
from .GXManufacturer import GXManufacturer
from .GXObisCode import GXObisCode
from .GXAuthentication import GXAuthentication
from .GXServerAddress import GXServerAddress
from .GXDLMSAttribute import GXDLMSAttribute
from .InactivityMode import InactivityMode
from .StartProtocolType import StartProtocolType
from .HDLCAddressType import HDLCAddressType
from ..enums import Authentication, DataType, ObjectType
if sys.version_info >= (3, 0):
    import urllib.request

#
class GXManufacturerCollection(list):
    #
    # * Find manufacturer settings by manufacturer id.
    #
    # @param id
    #            Manufacturer id.
    # found manufacturer or null.
    #
    def findByIdentification(self, id_):
        for it in self:
            if it.identification == id_:
                return it
        return None

    #
    # @param path
    #            Is this first run.
    # Settings directory.
    #
    @classmethod
    def isFirstRun(cls, path):
        if not os.path.isdir(path):
            os.mkdir(path)
            return True
        if not os.path.isfile(os.path.join(path, "files.xml")):
            return True
        return False

    #
    # Check if there are any updates available in Gurux www server.
    #
    # @param path
    #            Settings directory.
    # Returns true if there are any updates available.
    #
    @classmethod
    def isUpdatesAvailable(cls, path):
        if sys.version_info < (3, 0):
            return False
        # pylint: disable=broad-except
        if not os.path.isfile(os.path.join(path, "files.xml")):
            return True
        try:
            available = dict()
            for it in ET.parse(os.path.join(path, "files.xml")).iter():
                if it.tag == "File":
                    available[it.text] = datetime.datetime.strptime(it.attrib["Modified"], "%d-%m-%Y")

            path = NamedTemporaryFile()
            path.close()
            urllib.request.urlretrieve("https://www.gurux.fi/obis/files.xml", path.name)
            for it in ET.parse(path.name).iter():
                if it.tag == "File":
                    tmp = datetime.datetime.strptime(it.attrib["Modified"], "%d-%m-%Y")
                    if not it.text in available or available[it.text] != tmp:
                        return True
        except Exception as e:
            print(e)
            return True
        return False

    @classmethod
    def updateManufactureSettings(cls, directory):
        #
        # Update manufacturer settings from the Gurux www server.
        #
        # directory: Target directory.
        #
        if sys.version_info >= (3, 0):
            return
        if not os.path.isdir(directory):
            os.mkdir(directory)
            if not os.path.isdir(directory):
                return
        path = os.path.join(directory, "files.xml")
        urllib.request.urlretrieve("https://www.gurux.fi/obis/files.xml", path)
        for it in ET.parse(path).iter():
            if it.tag == "File":
                path = os.path.join(directory, it.text)
                urllib.request.urlretrieve("https://www.gurux.fi/obis/" + it.text, path)

    @classmethod
    def readManufacturerSettings(cls, manufacturers, path):
        # pylint: disable=broad-except
        manufacturers = []
        files = [f for f in listdir(path) if isfile(join(path, f))]
        if files:
            for it in files:
                if it.endswith(".obx"):
                    try:
                        manufacturers.append(cls.__parse(os.path.join(path, it)))
                    except Exception as e:
                        print(e)
                        continue

    #
    # Serialize manufacturer from the xml.
    #
    # @param in
    #            Input stream.
    # Serialized manufacturer.
    #
    @classmethod
    def __parse(cls, file):
        man = None
        authentication = None
        for it in ET.parse(file).iter():
            if it.tag == "GXManufacturer":
                man = GXManufacturer()
            elif it.tag == "GXObisCode":
                obisCode = GXObisCode()
                man.obisCodes.append(obisCode)
            elif it.tag == "GXAuthentication":
                authentication = GXAuthentication()
                man.settings.append(authentication)
            elif it.tag == "GXServerAddress":
                serveraddress = GXServerAddress()
                man.serverSettings.append(serveraddress)
            elif it.tag == "GXDLMSAttributeSettings":
                att = GXDLMSAttribute()
                obisCode.attributes.append(att)
            elif it.tag == "Identification":
                man.identification = it.text
            elif it.tag == "Name":
                man.name = it.text
            elif it.tag == "UseLN":
                man.useLogicalNameReferencing = bool(it.text)
            elif it.tag == "UseIEC47":
                man.useIEC47 = bool(it.text)
            elif it.tag == "ClientAddress":
                if authentication:
                    authentication.clientAddress = int(it.text)
            elif it.tag == "PhysicalAddress":
                if serveraddress:
                    serveraddress.physicalAddress = int(it.text)
            elif it.tag == "LogicalAddress":
                if serveraddress:
                    serveraddress.logicalAddress = int(it.text)
            elif it.tag == "Formula":
                if serveraddress:
                    serveraddress.formula = it.text
            elif it.tag == "HDLCAddress":
                if serveraddress:
                    serveraddress.HDLCAddress = (HDLCAddressType(int(it.text)))
            elif it.tag == "Selected":
                #  Old functionality.
                continue
            elif it.tag == "InactivityMode":
                man.inactivityMode = InactivityMode[it.text.upper()]
            elif it.tag == "Type":
                if authentication:
                    str_ = it.text.upper().replace("HIGH", "HIGH_")
                    if str_ == "HIGH_":
                        str_ = "HIGH"
                    authentication.type_ = Authentication[str_]
                else:
                    if it.text == "BinaryCodedDesimal":
                        att.type_ = DataType.BCD
                    elif it.text == "OctetString":
                        att.type_ = DataType.OCTET_STRING
                    else:
                        att.type_ = DataType[it.text.upper()]
            elif it.tag == "UIType":
                if it.text == "BinaryCodedDesimal":
                    att.uiType = DataType.BCD
                elif it.text == "OctetString":
                    att.uiType = DataType.OCTET_STRING
                else:
                    att.uiType = DataType[it.text.upper()]
            elif it.tag == "GXAuthentication":
                authentication = None
            elif it.tag == "LogicalName":
                obisCode.logicalName = it.text
            elif it.tag == "Description":
                obisCode.description = it.text
            elif it.tag == "ObjectType":
                obisCode.objectType = ObjectType(int(it.text))
            elif it.tag == "Interface":
                #  Old way.  ObjectType is used now.
                continue
            elif it.tag == "Index":
                att.index = int(it.text)
            elif it.tag == "StartProtocol":
                man.startProtocol = StartProtocolType[it.text]
            elif it.tag == "WebAddress":
                man.webAddress = it.text
            elif it.tag == "Info":
                man.info = it.text
        return man



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                   $Date$
#                   $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
from ..GXByteBuffer import GXByteBuffer

###Python 2 requires this
#pylint: disable=bad-option-value,old-style-class,too-few-public-methods
class GXDLMSImageActivateInfo:
    def __init__(self, size=0, identification=None, signature=None):
        """
        Constructor.

        size: Size.
        identification: Identification.
        signature: Signature.
        """
        self.size = size
        self.identification = identification
        self.signature = signature

    def __str__(self):
        sb = ""
        if GXByteBuffer.isAsciiString(self.identification):
            sb += str(self.identification)
        else:
            sb.append(GXByteBuffer.hex(self.identification, True))
        sb += " "
        if GXByteBuffer.isAsciiString(self.signature):
            sb += str(self.signature)
        else:
            sb += GXByteBuffer.hex(self.signature, True)
        sb += " "
        sb += str(self.size)
        return sb



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                   $Date$
#                   $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
from ..enums  import Conformance
from ..GXByteBuffer import GXByteBuffer

#pylint: disable=too-many-instance-attributes,too-few-public-methods, useless-object-inheritance
class GXxDLMSContextType(object):
    #
    # Constructor.
    #
    def __init__(self):
        # Server settings.
        self.__settings = None
        # Conformance.
        self.__conformance = Conformance.NONE
        # Maximum receive PDU size.
        self.__maxReceivePduSize = 0
        # Maximum Send PDU size.
        self.__maxSendPduSize = 0
        # DLMS Version Number.
        self.__dlmsVersionNumber = 0
        # Quality Of Service.
        self.qualityOfService = 0
        # CypheringInfo.
        self.cypheringInfo = []

    def __getConformance(self):
        if self.__settings:
            return self.__settings.proposedConformance
        return self.__conformance

    def __setConformance(self, value):
        if self.__settings:
            self.__settings.proposedConformance = value
        self.__conformance = value

    def __getMaxReceivePduSize(self):
        if self.__settings:
            return self.__settings.maxServerPDUSize
        return self.__maxReceivePduSize

    def __setMaxReceivePduSize(self, value):
        if self.__settings:
            self.__settings.maxServerPDUSize = value
        self.__maxReceivePduSize = value

    def __getMaxSendPduSize(self):
        if self.__settings:
            return self.__settings.maxServerPDUSize
        return self.__maxSendPduSize

    def __setMaxSendPduSize(self, value):
        if self.__settings:
            self.__settings.maxServerPDUSize = value
        self.__maxSendPduSize = value

    def __getDlmsVersionNumber(self):
        if self.__settings:
            return self.__settings.dlmsVersionNumber
        return self.__dlmsVersionNumber

    def __setDlmsVersionNumber(self, value):
        if self.__settings:
            self.__settings.dlmsVersionNumber = value
        self.__dlmsVersionNumber = value

    #Conformance
    conformance = property(__getConformance, __setConformance)

    maxReceivePduSize = property(__getMaxReceivePduSize, __setMaxReceivePduSize)

    maxSendPduSize = property(__getMaxSendPduSize, __setMaxSendPduSize)

    dlmsVersionNumber = property(__getDlmsVersionNumber, __setDlmsVersionNumber)

    def __str__(self):
        str_ = str(self.conformance) + " " + str(self.maxReceivePduSize) + " "
        str_ += str(self.maxSendPduSize) + " " + str(self.dlmsVersionNumber) + " "
        str_ += str(self.qualityOfService) + " " + GXByteBuffer.hex(self.cypheringInfo, True)
        return str_



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                   $Date$
#                   $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
from .GXDLMSObject import GXDLMSObject
from .IGXDLMSBase import IGXDLMSBase
from ..enums import ErrorCode
from ..internal._GXCommon import _GXCommon
from ..GXByteBuffer import GXByteBuffer
from ..enums import ObjectType, DataType
from .GXDLMSSpecialDay import GXDLMSSpecialDay

# pylint: disable=too-many-instance-attributes
class GXDLMSSpecialDaysTable(GXDLMSObject, IGXDLMSBase):
    """
    Online help:
    http://www.gurux.fi/Gurux.DLMS.Objects.GXDLMSSpecialDaysTable
    """

    def __init__(self, ln="0.0.11.0.0.255", sn=0):
        """
        Constructor.

        ln : Logical Name of the object.
        sn : Short Name of the object.
        """
        super(GXDLMSSpecialDaysTable, self).__init__(ObjectType.SPECIAL_DAYS_TABLE, ln, sn)
        # Special day entries.
        self.entries = list()

    #
    # Inserts a new entry in the table
    # If a special day with the same index or with the same date as an
    #      already
    # defined day is inserted, the old entry will be overwritten.
    #
    # @param client
    # DLMS Client.
    # @param entries
    # Inserted special day entries.
    # Generated byte array.
    def insert(self, client, entries):
        bb = GXByteBuffer()
        bb.setUInt8(DataType.ARRAY)
        _GXCommon.setObjectCount(len(entries), bb)
        for entry in entries:
            bb.setUInt8(DataType.STRUCTURE)
            bb.setUInt8(3)
            _GXCommon.setData(None, bb, DataType.UINT16, entry.index)
            _GXCommon.setData(None, bb, DataType.OCTET_STRING, entry.date)
            _GXCommon.setData(None, bb, DataType.UINT8, entry.dayId)
        return client.method(self, 1, bb.array(), DataType.ARRAY)

    #
    # Deletes an entry in the table.
    #
    # @param client
    # DLMS Client.
    # @param entry
    # Deleted special day item.
    # Generated byte array.
    def delete(self, client, entry):
        return client.method(self, 2, entry.index, DataType.UINT16)

    def getValues(self):
        return [self.logicalName,
                self.entries]

    def getAttributeIndexToRead(self, all_):
        attributes = list()
        #  LN is static and read only once.
        if all_ or not self.logicalName:
            attributes.append(1)
        #  Entries
        if all_ or not self.isRead(2):
            attributes.append(2)
        return attributes

    #
    # Returns amount of attributes.
    #
    def getAttributeCount(self):
        return 2

    #
    # Returns amount of methods.
    #
    def getMethodCount(self):
        return 2

    def getDataType(self, index):
        if index == 1:
            return DataType.OCTET_STRING
        if index == 2:
            return DataType.ARRAY
        raise ValueError("getDataType failed. Invalid attribute index.")

    #
    # Returns value of given attribute.
    #
    def getValue(self, settings, e):
        if e.index == 1:
            return _GXCommon.logicalNameToBytes(self.logicalName)
        if e.index == 2:
            data = GXByteBuffer()
            data.setUInt8(DataType.ARRAY)
            if not self.entries:
                _GXCommon.setObjectCount(0, data)
            else:
                cnt = len(self.entries)
                #  Add count
                _GXCommon.setObjectCount(cnt, data)
                for it in self.entries:
                    data.setUInt8(DataType.STRUCTURE)
                    data.setUInt8(3)
                    #  Count
                    _GXCommon.setData(settings, data, DataType.UINT16, it.index)
                    _GXCommon.setData(settings, data, DataType.OCTET_STRING, it.date)
                    _GXCommon.setData(settings, data, DataType.UINT8, it.dayId)
            return data
        e.error = ErrorCode.READ_WRITE_DENIED
        return None

    #
    # Set value of given attribute.
    #
    def setValue(self, settings, e):
        if e.index == 1:
            self.logicalName = _GXCommon.toLogicalName(e.value)
        elif e.index == 2:
            self.entries = []
            if e.value:
                for item in e.value:
                    it = GXDLMSSpecialDay()
                    it.index = item[0]
                    it.date = _GXCommon.changeType(settings, item[1], DataType.DATE)
                    it.dayId = item[2]
                    self.entries.append(it)
        else:
            e.error = ErrorCode.READ_WRITE_DENIED

    def invoke(self, settings, e):
        if e.index != 1 and e.index != 2:
            e.error = ErrorCode.READ_WRITE_DENIED
        else:
            items = list()
            if self.entries:
                items.append(self.entries)
            if e.index == 1:
                item = e.parameters
                it = GXDLMSSpecialDay()
                it.index = item[0]
                it.date = _GXCommon.changeType(settings, item[1], DataType.DATE)
                it.dayId = item[2]
                for item2 in items:
                    if item2.index == it.index:
                        items.remove(item2)
                        break
                items.append(it)
            elif e.index == 2:
                index = e.parameters
                for item in items:
                    if item.index == index:
                        items.remove(item)
                        break
            self.entries = items

    def load(self, reader):
        self.entries = []
        if reader.isStartElement("Entries", True):
            while reader.isStartElement("Entry", True):
                it = GXDLMSSpecialDay()
                it.index = reader.readElementContentAsInt("Index")
                it.date = reader.readElementContentAsDate("Date")
                it.dayId = reader.readElementContentAsInt("DayId")
                self.entries.append(it)
            reader.readEndElement("Entries")

    def save(self, writer):
        if self.entries:
            writer.writeStartElement("Entries")
            for it in self.entries:
                writer.writeStartElement("Entry")
                writer.writeElementString("Index", it.index)
                writer.writeElementString("Date", it.date)
                writer.writeElementString("DayId", it.dayId)
                writer.writeEndElement()
            writer.writeEndElement()



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                   $Date$
#                   $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
from .enums import ObjectType
#pylint: disable=bad-option-value,too-many-locals,
#cyclic-import,old-style-class,too-few-public-methods
from .objects.GXDLMSAssociationLogicalName import GXDLMSAssociationLogicalName
from .objects.GXDLMSObject import GXDLMSObject
from .objects.GXDLMSActionSchedule import GXDLMSActionSchedule
from .objects.GXDLMSActivityCalendar import GXDLMSActivityCalendar
from .objects.GXDLMSAssociationShortName import GXDLMSAssociationShortName
from .objects.GXDLMSAutoAnswer import GXDLMSAutoAnswer
from .objects.GXDLMSAutoConnect import GXDLMSAutoConnect
from .objects.GXDLMSClock import GXDLMSClock
from .objects.GXDLMSData import GXDLMSData
from .objects.GXDLMSDemandRegister import GXDLMSDemandRegister
from .objects.GXDLMSMacAddressSetup import GXDLMSMacAddressSetup
from .objects.GXDLMSRegister import GXDLMSRegister
from .objects.GXDLMSExtendedRegister import GXDLMSExtendedRegister
from .objects.GXDLMSGprsSetup import GXDLMSGprsSetup
from .objects.GXDLMSHdlcSetup import GXDLMSHdlcSetup
from .objects.GXDLMSIECLocalPortSetup import GXDLMSIECLocalPortSetup
from .objects.GXDLMSIecTwistedPairSetup import GXDLMSIecTwistedPairSetup
from .objects.GXDLMSIp4Setup import GXDLMSIp4Setup
from .objects.GXDLMSIp6Setup import GXDLMSIp6Setup
from .objects.GXDLMSMBusSlavePortSetup import GXDLMSMBusSlavePortSetup
from .objects.GXDLMSImageTransfer import GXDLMSImageTransfer
from .objects.GXDLMSSecuritySetup import GXDLMSSecuritySetup
from .objects.GXDLMSDisconnectControl import GXDLMSDisconnectControl
from .objects.GXDLMSLimiter import GXDLMSLimiter

from .objects.GXDLMSMBusClient import GXDLMSMBusClient
from .objects.GXDLMSModemConfiguration import GXDLMSModemConfiguration
from .objects.GXDLMSPppSetup import GXDLMSPppSetup
from .objects.GXDLMSProfileGeneric import GXDLMSProfileGeneric
from .objects.GXDLMSRegisterMonitor import GXDLMSRegisterMonitor
from .objects.GXDLMSRegisterActivation import GXDLMSRegisterActivation
from .objects.GXDLMSSapAssignment import GXDLMSSapAssignment
from .objects.GXDLMSSchedule import GXDLMSSchedule
from .objects.GXDLMSScriptTable import GXDLMSScriptTable
from .objects.GXDLMSSpecialDaysTable import GXDLMSSpecialDaysTable
from .objects.GXDLMSTcpUdpSetup  import GXDLMSTcpUdpSetup
from .objects.GXDLMSPushSetup import GXDLMSPushSetup
from .objects.GXDLMSMBusMasterPortSetup import GXDLMSMBusMasterPortSetup
from .objects.GXDLMSGSMDiagnostic import GXDLMSGSMDiagnostic
from .objects.GXDLMSAccount import GXDLMSAccount
from .objects.GXDLMSCredit import GXDLMSCredit
from .objects.GXDLMSCharge import GXDLMSCharge
from .objects.GXDLMSTokenGateway import GXDLMSTokenGateway
from .objects.GXDLMSParameterMonitor import GXDLMSParameterMonitor
from .objects.GXDLMSUtilityTables import GXDLMSUtilityTables
from .objects.GXDLMSLlcSscsSetup import GXDLMSLlcSscsSetup
from .objects.GXDLMSPrimeNbOfdmPlcPhysicalLayerCounters import GXDLMSPrimeNbOfdmPlcPhysicalLayerCounters
from .objects.GXDLMSPrimeNbOfdmPlcMacSetup import GXDLMSPrimeNbOfdmPlcMacSetup
from .objects.GXDLMSPrimeNbOfdmPlcMacFunctionalParameters import GXDLMSPrimeNbOfdmPlcMacFunctionalParameters
from .objects.GXDLMSPrimeNbOfdmPlcMacCounters import GXDLMSPrimeNbOfdmPlcMacCounters
from .objects.GXDLMSPrimeNbOfdmPlcMacNetworkAdministrationData import GXDLMSPrimeNbOfdmPlcMacNetworkAdministrationData
from .objects.GXDLMSPrimeNbOfdmPlcApplicationsIdentification import GXDLMSPrimeNbOfdmPlcApplicationsIdentification

class _GXObjectFactory:
    #Reserved for internal use.

    #
    # Constructor.
    def __init__(self):
        pass

    @classmethod
    def createObject(cls, ot):
        #pylint: disable=bad-option-value,redefined-variable-type
        #  If IC is manufacturer specific or unknown.
        if ot is None:
            raise ValueError("Invalid object type.")

        if ot == ObjectType.ACTION_SCHEDULE:
            ret = GXDLMSActionSchedule()
        elif ot == ObjectType.ACTIVITY_CALENDAR:
            ret = GXDLMSActivityCalendar()
        elif ot == ObjectType.ASSOCIATION_LOGICAL_NAME:
            ret = GXDLMSAssociationLogicalName()
        elif ot == ObjectType.ASSOCIATION_SHORT_NAME:
            ret = GXDLMSAssociationShortName()
        elif ot == ObjectType.AUTO_ANSWER:
            ret = GXDLMSAutoAnswer()
        elif ot == ObjectType.AUTO_CONNECT:
            ret = GXDLMSAutoConnect()
        elif ot == ObjectType.CLOCK:
            ret = GXDLMSClock()
        elif ot == ObjectType.DATA:
            ret = GXDLMSData()
        elif ot == ObjectType.DEMAND_REGISTER:
            ret = GXDLMSDemandRegister()
        elif ot == ObjectType.MAC_ADDRESS_SETUP:
            ret = GXDLMSMacAddressSetup()
        elif ot == ObjectType.REGISTER:
            ret = GXDLMSRegister()
        elif ot == ObjectType.EXTENDED_REGISTER:
            ret = GXDLMSExtendedRegister()
        elif ot == ObjectType.GPRS_SETUP:
            ret = GXDLMSGprsSetup()
        elif ot == ObjectType.IEC_HDLC_SETUP:
            ret = GXDLMSHdlcSetup()
        elif ot == ObjectType.IEC_LOCAL_PORT_SETUP:
            ret = GXDLMSIECLocalPortSetup()
        elif ot == ObjectType.IEC_TWISTED_PAIR_SETUP:
            ret = GXDLMSIecTwistedPairSetup()
        elif ot == ObjectType.IP4_SETUP:
            ret = GXDLMSIp4Setup()
        elif ot == ObjectType.IP6_SETUP:
            ret = GXDLMSIp6Setup()
        elif ot == ObjectType.MBUS_SLAVE_PORT_SETUP:
            ret = GXDLMSMBusSlavePortSetup()
        elif ot == ObjectType.IMAGE_TRANSFER:
            ret = GXDLMSImageTransfer()
        elif ot == ObjectType.SECURITY_SETUP:
            ret = GXDLMSSecuritySetup()
        elif ot == ObjectType.DISCONNECT_CONTROL:
            ret = GXDLMSDisconnectControl()
        elif ot == ObjectType.LIMITER:
            ret = GXDLMSLimiter()
        elif ot == ObjectType.MBUS_CLIENT:
            ret = GXDLMSMBusClient()
        elif ot == ObjectType.MODEM_CONFIGURATION:
            ret = GXDLMSModemConfiguration()
        elif ot == ObjectType.PPP_SETUP:
            ret = GXDLMSPppSetup()
        elif ot == ObjectType.PROFILE_GENERIC:
            ret = GXDLMSProfileGeneric()
        elif ot == ObjectType.REGISTER_MONITOR:
            ret = GXDLMSRegisterMonitor()
        elif ot == ObjectType.REGISTER_ACTIVATION:
            ret = GXDLMSRegisterActivation()
        elif ot == ObjectType.REGISTER_TABLE:
            ret = GXDLMSObject(ot)
        elif ot == ObjectType.ZIG_BEE_SAS_STARTUP:
            ret = GXDLMSObject(ot)
        elif ot == ObjectType.ZIG_BEE_SAS_JOIN:
            ret = GXDLMSObject(ot)
        elif ot == ObjectType.SAP_ASSIGNMENT:
            ret = GXDLMSSapAssignment()
        elif ot == ObjectType.SCHEDULE:
            ret = GXDLMSSchedule()
        elif ot == ObjectType.SCRIPT_TABLE:
            ret = GXDLMSScriptTable()
        elif ot == ObjectType.SPECIAL_DAYS_TABLE:
            ret = GXDLMSSpecialDaysTable()
        elif ot == ObjectType.STATUS_MAPPING:
            ret = GXDLMSObject(ot)
        elif ot == ObjectType.TCP_UDP_SETUP:
            ret = GXDLMSTcpUdpSetup()
        elif ot == ObjectType.ZIG_BEE_SAS_APS_FRAGMENTATION:
            ret = GXDLMSObject(ot)
        elif ot == ObjectType.UTILITY_TABLES:
            ret = GXDLMSUtilityTables()
        elif ot == ObjectType.PUSH_SETUP:
            ret = GXDLMSPushSetup()
        elif ot == ObjectType.MBUS_MASTER_PORT_SETUP:
            ret = GXDLMSMBusMasterPortSetup()
        elif ot == ObjectType.GSM_DIAGNOSTIC:
            ret = GXDLMSGSMDiagnostic()
        elif ot == ObjectType.ACCOUNT:
            ret = GXDLMSAccount()
        elif ot == ObjectType.CREDIT:
            ret = GXDLMSCredit()
        elif ot == ObjectType.CHARGE:
            ret = GXDLMSCharge()
        elif ot == ObjectType.TOKEN_GATEWAY:
            ret = GXDLMSTokenGateway()
        elif ot == ObjectType.PARAMETER_MONITOR:
            ret = GXDLMSParameterMonitor()
        elif ot == ObjectType.LLC_SSCS_SETUP:
            ret = GXDLMSLlcSscsSetup()
        elif ot == ObjectType.PRIME_NB_OFDM_PLC_PHYSICAL_LAYER_COUNTERS:
            ret = GXDLMSPrimeNbOfdmPlcPhysicalLayerCounters()
        elif ot == ObjectType.PRIME_NB_OFDM_PLC_MAC_SETUP:
            ret = GXDLMSPrimeNbOfdmPlcMacSetup()
        elif ot == ObjectType.PRIME_NB_OFDM_PLC_MAC_FUNCTIONAL_PARAMETERS:
            ret = GXDLMSPrimeNbOfdmPlcMacFunctionalParameters()
        elif ot == ObjectType.PRIME_NB_OFDM_PLC_MAC_COUNTERS:
            ret = GXDLMSPrimeNbOfdmPlcMacCounters()
        elif ot == ObjectType.PRIME_NB_OFDM_PLC_MAC_NETWORK_ADMINISTRATION_DATA:
            ret = GXDLMSPrimeNbOfdmPlcMacNetworkAdministrationData()
        elif ot == ObjectType.PRIME_NB_OFDM_PLC_APPLICATIONS_IDENTIFICATION:
            ret = GXDLMSPrimeNbOfdmPlcApplicationsIdentification()
        else:
            ret = GXDLMSObject(ot)
        return ret



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                   $Date$
#                   $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
from .GXDLMSObject import GXDLMSObject
from .IGXDLMSBase import IGXDLMSBase
from ..enums import ErrorCode
from ..internal._GXCommon import _GXCommon
from ..GXDateTime import GXDateTime
from ..enums import ObjectType, DataType
from .enums import CreditConfiguration, CreditType, CreditStatus
from ..GXBitString import GXBitString

# pylint: disable=too-many-instance-attributes
class GXDLMSCredit(GXDLMSObject, IGXDLMSBase):
    """
    Online help:
    http://www.gurux.fi/Gurux.DLMS.Objects.GXDLMSCredit
    """

    def __init__(self, ln="0.0.19.10.0.255", sn=0):
        """
        Constructor.

        ln : Logical Name of the object.
        sn : Short Name of the object.
        """
        GXDLMSObject.__init__(self, ObjectType.CREDIT, ln, sn)
        self.creditConfiguration = CreditConfiguration.NONE
        self.type_ = CreditType.TOKEN
        self.status = CreditStatus.ENABLED
        self.currentCreditAmount = 0
        self.priority = 0
        self.warningThreshold = 0
        self.limit = 0
        self.presetCreditAmount = 0
        self.creditAvailableThreshold = 0
        self.period = None

    def getValues(self):
        return [self.logicalName,
                self.currentCreditAmount,
                self.type_,
                self.priority,
                self.warningThreshold,
                self.limit,
                self.creditConfiguration,
                self.status,
                self.presetCreditAmount,
                self.creditAvailableThreshold,
                self.period]

    #
    # Returns collection of attributes to read.  If attribute is static and
    # already read or device is returned HW error it is not returned.
    #
    def getAttributeIndexToRead(self, all_):
        attributes = list()
        #  LN is static and read only once.
        if all_ or not self.logicalName:
            attributes.append(1)
        #  CurrentCreditAmount
        if all_ or self.canRead(2):
            attributes.append(2)
        #  Type
        if all_ or self.canRead(3):
            attributes.append(3)
        #  Priority
        if all_ or self.canRead(4):
            attributes.append(4)
        #  WarningThreshold
        if all_ or self.canRead(5):
            attributes.append(5)
        #  Limit
        if all_ or self.canRead(6):
            attributes.append(6)
        #  creditConfiguration
        if all_ or self.canRead(7):
            attributes.append(7)
        #  Status
        if all_ or self.canRead(8):
            attributes.append(8)
        #  PresetCreditAmount
        if all_ or self.canRead(9):
            attributes.append(9)
        #  CreditAvailableThreshold
        if all_ or self.canRead(10):
            attributes.append(10)
        #  Period
        if all_ or self.canRead(11):
            attributes.append(11)
        return attributes

    #
    # Returns amount of attributes.
    #
    def getAttributeCount(self):
        return 11

    #
    # Returns amount of methods.
    #
    def getMethodCount(self):
        return 3

    def getDataType(self, index):
        if index == 1:
            ret = DataType.OCTET_STRING
        elif index == 2:
            ret = DataType.INT32
        elif index == 3:
            ret = DataType.ENUM
        elif index == 4:
            ret = DataType.UINT8
        elif index == 5:
            ret = DataType.INT32
        elif index == 6:
            ret = DataType.INT32
        elif index == 7:
            ret = DataType.BITSTRING
        elif index == 8:
            ret = DataType.ENUM
        elif index == 9:
            ret = DataType.INT32
        elif index == 10:
            ret = DataType.INT32
        elif index == 11:
            ret = DataType.OCTET_STRING
        else:
            raise ValueError("getDataType failed. Invalid attribute index.")
        return ret
    #
    # Returns value of given attribute.
    #
    def getValue(self, settings, e):
        #pylint: disable=bad-option-value,redefined-variable-type
        if e.index == 1:
            ret = _GXCommon.logicalNameToBytes(self.logicalName)
        elif e.index == 2:
            ret = self.currentCreditAmount
        elif e.index == 3:
            ret = self.type_
        elif e.index == 4:
            ret = self.priority
        elif e.index == 5:
            ret = self.warningThreshold
        elif e.index == 6:
            ret = self.limit
        elif e.index == 7:
            ret = GXBitString.toBitString(self.creditConfiguration, 5)
        elif e.index == 8:
            ret = self.status
        elif e.index == 9:
            ret = self.presetCreditAmount
        elif e.index == 10:
            ret = self.creditAvailableThreshold
        elif e.index == 11:
            ret = self.period
        else:
            e.error = ErrorCode.READ_WRITE_DENIED
        return ret

    #
    # Set value of given attribute.
    #
    def setValue(self, settings, e):
        if e.index == 1:
            self.logicalName = _GXCommon.toLogicalName(e.value)
        elif e.index == 2:
            self.currentCreditAmount = e.value
        elif e.index == 3:
            self.type_ = e.value
        elif e.index == 4:
            self.priority = e.value
        elif e.index == 5:
            self.warningThreshold = e.value
        elif e.index == 6:
            self.limit = e.value
        elif e.index == 7:
            self.creditConfiguration = CreditConfiguration(e.value.toInteger())
        elif e.index == 8:
            self.status = e.value
        elif e.index == 9:
            self.presetCreditAmount = e.value
        elif e.index == 10:
            self.creditAvailableThreshold = e.value
        elif e.index == 11:
            if e.value is None:
                self.period = GXDateTime()
            else:
                tmp = None
                if isinstance(e.value, bytearray):
                    tmp = _GXCommon.changeType(settings, e.value, DataType.DATETIME)
                else:
                    tmp = e.value
                self.period = tmp
        else:
            e.error = ErrorCode.READ_WRITE_DENIED

    def load(self, reader):
        self.currentCreditAmount = reader.readElementContentAsInt("CurrentCreditAmount")
        self.type_ = reader.readElementContentAsInt("Type")
        self.priority = reader.readElementContentAsInt("Priority")
        self.warningThreshold = reader.readElementContentAsInt("WarningThreshold")
        self.limit = reader.readElementContentAsInt("Limit")
        self.creditConfiguration = CreditConfiguration(reader.readElementContentAsInt("CreditConfiguration"))
        self.status = reader.readElementContentAsInt("Status")
        self.presetCreditAmount = reader.readElementContentAsInt("PresetCreditAmount")
        self.creditAvailableThreshold = reader.readElementContentAsInt("CreditAvailableThreshold")
        self.period = reader.readElementContentAsDateTime("Period")

    def save(self, writer):
        writer.writeElementString("CurrentCreditAmount", self.currentCreditAmount)
        writer.writeElementString("Type", int(self.type_))
        writer.writeElementString("Priority", self.priority)
        writer.writeElementString("WarningThreshold", self.warningThreshold)
        writer.writeElementString("Limit", self.limit)
        writer.writeElementString("CreditConfiguration", int(self.creditConfiguration))
        writer.writeElementString("Status", int(self.status))
        writer.writeElementString("PresetCreditAmount", self.presetCreditAmount)
        writer.writeElementString("CreditAvailableThreshold", self.creditAvailableThreshold)
        writer.writeElementString("Period", self.period)
